
//  PacmanApplication.java  Copyright (c) Kari Laitinen

//  http://www.naturalprogramming.com/

//  2014-11-25  This file was created.
//  2014-11-25  Last modification

//  This application displays a "Pacman" whose mouth opens and closes
//  constantly. This is thus yet another example of simple animation.

import java.awt.* ;
import javax.swing.* ;
import java.awt.event.* ; // WindowListener, WindowEvent


class PacmanPanel extends JPanel implements Runnable
{
   Thread  animation_thread  =  null ;

   boolean  thread_must_be_executed  =  false ;

   int[]  pacman_mouth_angles  =  { 90, 75, 60, 45, 30, 15,
                                    0,  15, 30, 45, 60, 75 } ;

   int  mouth_angle_index  =  0 ;

   public PacmanPanel()
   {
      setBackground( new Color( 255, 255, 110 ) ) ; // light yellow
   }

   public void start_animation_thread()
   {
      if ( animation_thread  ==  null )
      {
         animation_thread  =  new  Thread( this ) ;
         thread_must_be_executed  =  true ;

         animation_thread.start() ;
      }
   }

   public void stop_animation_thread()
   {
      if ( animation_thread  !=  null )
      {
         animation_thread.interrupt() ;

         thread_must_be_executed  =  false ;

         animation_thread  =  null ;
      }
   }

   public void run()
   {
      while ( thread_must_be_executed  ==  true )
      {
         try
         {
            Thread.sleep( 50 ) ;
         }
         catch ( InterruptedException  caught_exception )
         {
            thread_must_be_executed  =  false ;
         }

         mouth_angle_index  ++  ;

         if ( mouth_angle_index  >=  pacman_mouth_angles.length )
         {
            mouth_angle_index  =  0  ;
         }

         repaint() ;
      }
   }

   public void paintComponent( Graphics graphics )
   {
      super.paintComponent( graphics ) ;

      int pacman_position_x  =  getSize().width / 2  -  60 ;
      int pacman_position_y  =  getSize().height / 2  -  50 ;

      int pacman_mouth_angle  =  pacman_mouth_angles[ mouth_angle_index ] ;

      graphics.fillArc(  pacman_position_x,
                         pacman_position_y,
                         120, 100,
                         pacman_mouth_angle / 2,
                         360 - pacman_mouth_angle ) ;

   }
}


class PacmanFrame extends JFrame implements WindowListener
{
   static final int FRAME_WIDTH   =  600 ;
   static final int FRAME_HEIGHT  =  580 ;

   PacmanPanel pacman_panel ;

   public PacmanFrame()
   {
      setTitle( "PacmanApplication.java" ) ;
      setSize( FRAME_WIDTH, FRAME_HEIGHT ) ;

      pacman_panel = new PacmanPanel() ;

      Container content_pane = getContentPane() ;
      content_pane.add( pacman_panel ) ;

      addWindowListener( this ) ;
   }

   public void windowClosing( WindowEvent event )
   {
      pacman_panel.stop_animation_thread() ;
      setVisible( false ) ;
      System.exit( 0 ) ;
   }

   public void windowActivated( WindowEvent event )
   {
   }

   public void windowDeactivated( WindowEvent event )
   {
   }


   public void windowClosed( WindowEvent event )
   {
      pacman_panel.stop_animation_thread() ;
   }

   public void windowDeiconified( WindowEvent event )
   {
      pacman_panel.start_animation_thread() ;
   }

   public void windowIconified( WindowEvent event )
   {
      pacman_panel.stop_animation_thread() ;
   }

   public void windowOpened( WindowEvent event )
   {
      pacman_panel.start_animation_thread() ;
   }

}


public class PacmanApplication
{
   public static void main( String[] not_in_use )
   {
      PacmanFrame pacman_frame = new PacmanFrame() ;

      pacman_frame.setVisible( true ) ;
   }
}

